/**
 * Created by andre on 20/08/2016.
 */
var app = angular.module('starter')

app.controller('eventosCtrl', function($scope, eventoService, Filtro, ConnectivityMonitor){

    // propriedade para mostrar o ícone e a mensagem de evento não
    // encontrado.
    $scope.eventNotFound = false; 

    // propriedade do loading
    $scope.showLoading = true;

    // propriedade de apresentação do filtro
    $scope.showFiltro = false;

    // lado que a timeline vai ser apresentada
    $scope.side = "left"; 

    $scope.currentData = Date.now();

    // instância do filtro
    var filtro = Filtro.getInstance();

    // função que busca os eventos. Quando finalizado a busca
    // deverá ocultar o loading.
    buscarEventos = function() {
        eventoService.getEventos(filtro).then(function(dados) {
            $scope.eventos = dados;

            // caso não tenha encontrado nenhum evento vai ser apresentado
            // a mensagem de evento não encontrado. Caso contrário deverá 
            // mostrar o ícone de filtro.
            if(dados != null) {
                if(dados.length == 0) {
                    $scope.eventNotFound = true;
                } else {
                    $scope.showFiltro = true;
                }                
            }
        }).finally(function(){
            // ocultando o loading
            $scope.showLoading = false;
        })
    }

    ConnectivityMonitor.startWatching(); //assistindo a conectividade
    buscarEventos();

});

app.controller('filtrosCtrl', function($scope, $state, $ionicLoading, $ionicHistory, tipoEventoService, 
    regiaoService, filtroService, Filtro, ConnectivityMonitor){

    var filtro = Filtro.getInstance();

    // propriedade do botão 'Todos' true = marcado, false = desmarcado.
    $scope.btTodos = {
        isClicked: true,
        show: false // mostra o botão
    }; 

    // função que busca as regiões e os seus correspondentes tipos de evento.
    buscarRegioes = function() {
        regiaoService.getRegioes().then(function(dados){
            $scope.regioes = dados;
            $scope.itemSelected = filtro.regiao;

            // se o filtro não esteja preenchido será feito uma nova busca.
            // Caso contrário, não deverá ser feito uma nova busca.
            if(filtro.tiposEvento.length == 0) {
                $scope.buscarTiposEvento($scope.itemSelected);
            } else {
                $scope.eventos = filtro.tiposEvento;
                verificarChecked();
                verificarBtTodos(); // marca ou desmarca o botão 'Todos'
                $scope.btTodos.show = true;

                $ionicLoading.hide();
            } 
        });
    }

    // função que verifica quais são os tipos de eventos que estão marcados (checked).
    verificarChecked = function() {
        angular.forEach(filtro.tiposEvento, function(tipo) {
            tipo.isClicked = false;
            angular.forEach(filtro.tiposEventoChecked, function(tipoChecked){
                if(tipo == tipoChecked){
                    tipo.isClicked = true;
                    return
                } 
            });
        });
    }

    // função que busca os tipos de evento. Se fornecido a região
    // deverá retornar os tipos de evento da mesma. Caso contrário
    // deverá retornar todos os tipos de evento ativos.
    // @param itemSelected - região selecionada.
    $scope.buscarTiposEvento = function(itemSelected) {
        $ionicLoading.show();
        regiaoSelected = null;
        var codRegiao = null;
        if (itemSelected) {
            regiaoSelected = itemSelected;
            codRegiao = itemSelected.id;
        }
        tipoEventoService.getTiposEvento(codRegiao).then(function(dados) {
            addProperty(dados);
            $scope.eventos = dados;

            if (dados != null && dados.length > 0) {
                $scope.btTodos.show = true;
                // caso a consulta resulte em todas as regiões, os ensaios locais
                // e as santa ceias não serão marcados.
                if(codRegiao) 
                    $scope.marcarTodos(true);
                else 
                    marcacaoDefault();
            }

        }).finally(function() {
            $ionicLoading.hide();
        });
    }

    // função que adiciona a propriedade isClicked nos eventos.
    // @param eventos - lista de eventos.
    addProperty = function(eventos) {
        var property = {isClicked:true};
        angular.forEach(eventos, function(evento) {
            angular.merge(evento, property);
        });
    }

    // função que desmarca os Ensaios Locais e Santas Ceias.
    marcacaoDefault = function() {
        angular.forEach($scope.eventos, function(evento) {
            if(evento.descricao == "Ensaio Local" || evento.descricao == "Santa Ceia") {
                evento.isClicked = false;
                $scope.btTodos.isClicked = false;
            } else {
                evento.isClicked = true;
            }
        });
    }

    // função que marca e desmarca todos os botões.
    // @param isClicked - true - marcado, false - desmarcado.
    $scope.marcarTodos = function(isClicked) {
        $scope.btTodos.isClicked = isClicked;
        angular.forEach($scope.eventos, function(evento) {
            evento.isClicked = $scope.btTodos.isClicked;
        });
    }

    // função que marca e desmarca um determinado evento.
    // @param eventoAtual - evento que será marcado ou desmarcado.
    // @param isClicked - true - marcado, false - desmarcado.
    $scope.marcarUnico = function(eventoAtual, isClicked) {
        eventoAtual.isClicked = isClicked;
        if(eventoAtual.isClicked == false) {
            $scope.btTodos.isClicked = false;
        } else {
            // caso todos os eventos foram marcados deverá marcar o 
            // botão todos.
            verificarBtTodos();
        }
    }

    // função que verifica se todos os botões estão marcados. Caso seja
    // verdade o botão 'Todos' será marcado
    verificarBtTodos = function() {
        $scope.btTodos.isClicked = true;
        angular.forEach($scope.eventos, function(evento) {
            if (evento.isClicked == false) {
                $scope.btTodos.isClicked = false;
                return
            }                        
        });
    }

    // função que preenche o filtroService. Caso o usuário
    // não tenha selecionado no mínimo um tipo de evento 
    // deverá ser apresentado um alerta.
    $scope.filtrar = function() {
        
        var aux = [];
        angular.forEach($scope.eventos, function(tipo) {
            if(tipo.isClicked) {
                aux.push(tipo)
            }
        });

        if(aux.length != 0) {
            filtro.regiao = regiaoSelected;
            filtro.tiposEvento = $scope.eventos;
            filtro.tiposEventoChecked = aux;            

            // limpando o cache antes de ir para a view.
            $ionicHistory.clearCache().then(function(){ $state.go('eventos')})
        } else {
            swal({
                title: "",
                text: "Selecione um evento!",
                type: "warning",
                animation: false
            });
        }
    }

    $ionicLoading.show();

    ConnectivityMonitor.startWatching(); //assistindo a conectividade
    buscarRegioes();

});
