var app = angular.module('starter');

var URL = "http://138.197.227.14/br.com.ccbeventos.rest/";

var config = {timeout:10000};

app.service('eventoService', function ($http, $state) {

    // função que retorna os eventos ativos que se encontram no DF.
    return {
        getEventos: function (filtro) {
        	var params;
        	// se o filtro está vazio deverá passar um json vazio. Caso contrário deverá passar o filtro.
        	if(filtro.regiao == null && filtro.tiposEvento.length == 0 && filtro.tiposEventoChecked.length == 0) {
        		params = JSON.stringify({});
        	} else {
        		params = JSON.stringify(filtro);
        	}

            return $http.post(URL + 'eventos', params, config).then(function successCallback(response) {
                console.log(response);
                return response.data;
            }, function errorCallback(response) {
        		    handleErrors($state, response.status);
            });
        }
    }

});

app.service('tipoEventoService', function ($http, $state) {

    // função que retorna os tipos de evento ativos em uma determinada região.
    return {
        getTiposEvento: function (codRegiao) {
            codRegiao ? aux = URL + 'tiposevento/regiao?codregiao=' + codRegiao : aux = URL + 'tiposevento';
            return $http.get(aux, config).then(function successCallback(response) {
                console.log(response);
                return response.data;
            }, function errorCallback(response) {
        		    handleErrors($state, response.status);
            });
        }
    }

});

app.service('regiaoService', function ($http, $state) {

    // função que retorna as regiões do DF que possuem um ou mais eventos.
    return {
        getRegioes: function () {
            return $http.get(URL + 'regioes', config).then(function successCallback(response) {
                console.log(response);
                return response.data;
            }, function errorCallback(response) {
        		    handleErrors($state, response.status);
            });
        
}    }
});

app.factory('filtroService', function() {

	 var regiao = {};
     var tiposEvento = [];
     var tiposEventoChecked = [];

	 function set(regiao, tiposEvento) {
	   	this.regiao = regiao;
	   	this.tiposEvento = tiposEvento;

	   	tiposEventoChecked = [];
	   	angular.forEach(tiposEvento, function(tipo) {
	 		if(tipo.isClicked) {
	 			tiposEventoChecked.push(tipo);
	 		}
	 	});

	 	this.tiposEventoChecked = tiposEventoChecked;
	 }

	 function get() {
	 	return this;
	 }
	 
	return {
	  set: set,
	  get: get
	} 
});

app.factory('Filtro', function() {

	var Filtro = (function () {
	    var instance;

	    function createInstance() {
	        var filtro = {
	        	regiao : null,
	        	tiposEvento : [],
	        	tiposEventoChecked : []
	        };
	        return filtro;
	    }
	 
	    return {
	        getInstance: function () {
	            if (!instance) {
	                instance = createInstance();
	            }
	            return instance;
	        }
	    };
	})();

	return Filtro;

});

app.factory('ConnectivityMonitor', function($rootScope, $cordovaNetwork, $state){
 
  return {
    startWatching: function(){
        if(ionic.Platform.isWebView()){
 
 		  //propriedade que evita executar a função offline duas vezes
 		  var internetConnected = true; 
          $rootScope.$on('$cordovaNetwork:online', function(event, networkState){
            internetConnected = true;
            $state.go($state.current, {}, {reload: true});// recarregando a view atual
        	  swal.close(); // fechando o alerta
          });
 
          $rootScope.$on('$cordovaNetwork:offline', function(event, networkState){
            if(!internetConnected ) return;
  	        internetConnected = false;
  	        showMsgNoConnection();
          });
        }
        else {
          window.addEventListener("online", function(e) {
            console.log("went online");
          }, false);    
 
          window.addEventListener("offline", function(e) {
            console.log("went offline");
          }, false);  
        }       
    }
  }
});

// função que manipula os erros.
// @param $state - estado da view
// @param status - código do erro
function handleErrors($state, status){
	switch(status) {
		case -1 :
      // verificando se possui alguma conexão
      if(window.Connection) {
  			//servidor fora do ar
  			showMsgProblemConnection($state);
      }
			break
		default:
			showMsgDefaultProblem($state);
	}
}

// função que apresenta a mensagem de quando não há conexão com a Internet.
function showMsgNoConnection(){
	swal({
		title: "Offline!",
		text: "Fique online e aguarde...",
		imageUrl: "images/no-signal.svg",
		animation: false
	});
	swal.disableButtons();
}

// função que apresenta a mensagem de problemas com o servidor. Ao clicar
// no botão ok o dispositivo irá recarregar a view.
function showMsgProblemConnection($state) {
	swal({
		title: "Opss!",
		text: "Não foi possível conectar ao servidor. Verifique sua conexão e tente novamente!",
		imageUrl: "images/no-database.svg",
		animation: false
	}, function(){
		$state.go($state.current, {}, {reload: true});
	});
}

// função que apresenta uma mensagem geral dos problemas ocorridos na aplicação.
// Ao clicar no botão ok irá recarregar a view  atual.
function showMsgDefaultProblem($state) {
	swal({
		title: "Erro!",
		text: "Houve algum problema. Se continuar reinicie o app.",
		type: "error",
		animation: false
	}, function(){
		$state.go($state.current, {}, {reload: true});
	});
}